@@ -45,22 +45,30 @@
 
 /* Only include this when using the file browser */
 
-#ifdef USE_BROWSE
+#ifdef FEAT_BROWSE
 
 /* Weird complication: for "make lint" Text.h doesn't combine with Xm.h */
-#if defined(USE_GUI_MOTIF) && defined(FMT8BIT)
+#if defined(FEAT_GUI_MOTIF) && defined(FMT8BIT)
 # undef FMT8BIT
 #endif
 
-#include "gui_at_sb.h"
+#ifndef FEAT_GUI_NEXTAW
+# include "gui_at_sb.h"
+#endif
 
 /***************** SFinternal.h */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
-#include <X11/Xaw/Text.h>
-#include <X11/Xaw/AsciiText.h>
+#ifdef FEAT_GUI_NEXTAW
+# include <X11/neXtaw/Text.h>
+# include <X11/neXtaw/AsciiText.h>
+# include <X11/neXtaw/Scrollbar.h>
+#else
+# include <X11/Xaw/Text.h>
+# include <X11/Xaw/AsciiText.h>
+#endif
 
 #define SEL_FILE_CANCEL		-1
 #define SEL_FILE_OK		0
@@ -103,6 +111,7 @@ static Widget	selFile,
 		selFileHScrolls[3],
 		selFileLists[3],
 		selFileOK,
+		selFileCancel,
 		selFilePrompt,
 		selFileVScrolls[3];
 
@@ -147,7 +156,11 @@ static XtAppContext SFapp;
 
 static int	SFpathScrollWidth, SFvScrollHeight, SFhScrollWidth;
 
+#ifdef FEAT_XFONTSET
+static char	SFtextBuffer[MAXPATHL*sizeof(wchar_t)];
+#else
 static char	SFtextBuffer[MAXPATHL];
+#endif
 
 static int	SFbuttonPressed = 0;
 
@@ -155,6 +168,8 @@ static XtIntervalId SFdirModTimerId;
 
 static int	(*SFfunc)();
 
+static int	SFstatus = SEL_FILE_NULL;
+
 /***************** static functions */
 
 static void SFsetText __ARGS((char *path));
@@ -176,16 +191,17 @@ static void SFenterList __ARGS((Widget w, int n, XEnterWindowEvent *event));
 static void SFleaveList __ARGS((Widget w, int n, XEvent *event));
 static void SFmotionList __ARGS((Widget w, int n, XMotionEvent *event));
 static void SFvFloatSliderMovedCallback __ARGS((Widget w, XtPointer n, XtPointer fnew));
-static void SFvSliderMovedCallback __ARGS((Widget w, int n, int new));
+static void SFvSliderMovedCallback __ARGS((Widget w, int n, int nw));
 static void SFvAreaSelectedCallback __ARGS((Widget w, XtPointer n, XtPointer pnew));
-static void SFhSliderMovedCallback __ARGS((Widget w, XtPointer n, XtPointer new));
+static void SFhSliderMovedCallback __ARGS((Widget w, XtPointer n, XtPointer nw));
 static void SFhAreaSelectedCallback __ARGS((Widget w, XtPointer n, XtPointer pnew));
-static void SFpathSliderMovedCallback __ARGS((Widget w, XtPointer client_data, XtPointer new));
+static void SFpathSliderMovedCallback __ARGS((Widget w, XtPointer client_data, XtPointer nw));
 static void SFpathAreaSelectedCallback __ARGS((Widget w, XtPointer client_data, XtPointer pnew));
 static Boolean SFworkProc __ARGS((void));
 static int SFcompareEntries __ARGS((const void *p, const void *q));
 static void SFprepareToReturn __ARGS((void));
 static void SFcreateWidgets __ARGS((Widget toplevel, char *prompt, char *ok, char *cancel));
+static void SFsetColors __ARGS((guicolor_T bg, guicolor_T fg, guicolor_T scroll_bg, guicolor_T scrollfg));
 
 /***************** xstat.h */
 
@@ -247,8 +263,8 @@ static void SFfree __ARGS((int i));
 SFfree(i)
     int	i;
 {
-    register SFDir	*dir;
-    register int	j;
+    SFDir	*dir;
+    int		j;
 
     dir = &(SFdirs[i]);
 
@@ -281,7 +297,7 @@ static void SFunreadableDir __ARGS((SFDir *dir));
 SFunreadableDir(dir)
     SFDir	*dir;
 {
-    char	*cannotOpen = "<cannot open> ";
+    char	*cannotOpen = _("<cannot open> ");
 
     dir->entries = (SFEntry *) XtMalloc(sizeof(SFEntry));
     dir->entries[0].statDone = 1;
@@ -371,19 +387,19 @@ SFexpand(str)
     XtFree(growing);
 }
 
-static int SFfindFile __ARGS((SFDir *dir, register char *str));
+static int SFfindFile __ARGS((SFDir *dir, char *str));
 
     static int
 SFfindFile(dir, str)
-    SFDir		*dir;
-    register char	*str;
+    SFDir	*dir;
+    char	*str;
 {
-    register int	i, last, max;
-    register char	*name, save;
-    SFEntry		*entries;
-    int			len;
-    int			begin, end;
-    int			result;
+    int		i, last, max;
+    char	*name, save;
+    SFEntry	*entries;
+    int		len;
+    int		begin, end;
+    int		result;
 
     len = strlen(str);
 
@@ -757,11 +773,18 @@ SFupdatePath()
 
     if ((SFdirPtr != SFdirPtrSave) || (SFdirEnd != SFdirEndSave))
     {
+#ifdef FEAT_GUI_NEXTAW
+	XawScrollbarSetThumb( selFileHScroll,
+		(float) (((double) SFdirPtr) / SFdirEnd),
+		(float) (((double) ((SFdirEnd < 3) ? SFdirEnd : 3)) /
+			 SFdirEnd));
+#else
 	vim_XawScrollbarSetThumb( selFileHScroll,
 		(float) (((double) SFdirPtr) / SFdirEnd),
 		(float) (((double) ((SFdirEnd < 3) ? SFdirEnd : 3)) /
 			 SFdirEnd),
 		(double)SFdirEnd);
+#endif
     }
 
     if (SFdirPtr != SFdirPtrSave)
@@ -782,6 +805,18 @@ SFupdatePath()
 	}
 }
 
+#ifdef XtNinternational
+    static int
+WcsLen(p)
+    wchar_t *p;
+{
+    int i = 0;
+    while (*p++ != 0)
+	i++;
+    return i;
+}
+#endif
+
     static void
 SFsetText(path)
     char	*path;
@@ -793,23 +828,38 @@ SFsetText(path)
     text.ptr = path;
     text.format = FMT8BIT;
 
+#ifdef XtNinternational
+    if ((unsigned long)_XawTextFormat((TextWidget)selFileField) == XawFmtWide)
+    {
+	XawTextReplace(selFileField, (XawTextPosition)0,
+				    (XawTextPosition)WcsLen((wchar_t *)&SFtextBuffer[0]), &text);
+	XawTextSetInsertionPoint(selFileField,
+					   (XawTextPosition)WcsLen((wchar_t *)&SFtextBuffer[0]));
+    }
+    else
+    {
+	XawTextReplace(selFileField, (XawTextPosition)0,
+				    (XawTextPosition)strlen(SFtextBuffer), &text);
+	XawTextSetInsertionPoint(selFileField,
+					   (XawTextPosition)strlen(SFtextBuffer));
+    }
+#else
     XawTextReplace(selFileField, (XawTextPosition)0,
 				(XawTextPosition)strlen(SFtextBuffer), &text);
     XawTextSetInsertionPoint(selFileField,
 				       (XawTextPosition)strlen(SFtextBuffer));
+#endif
 }
 
-/* ARGSUSED */
     static void
 SFbuttonPressList(w, n, event)
-    Widget		w;
-    int			n;
-    XButtonPressedEvent	*event;
+    Widget		w UNUSED;
+    int			n UNUSED;
+    XButtonPressedEvent	*event UNUSED;
 {
     SFbuttonPressed = 1;
 }
 
-/* ARGSUSED */
     static void
 SFbuttonReleaseList(w, n, event)
     Widget		 w;
@@ -842,7 +892,7 @@ SFcheckDir(n, dir)
     struct stat	statBuf;
     int		i;
 
-    if ((!stat(".", &statBuf)) && (statBuf.st_mtime != dir->mtime))
+    if ((!mch_stat(".", &statBuf)) && (statBuf.st_mtime != dir->mtime))
     {
 	/*
 	 * If the pointer is currently in the window that we are about
@@ -906,7 +956,7 @@ SFcheckFiles(dir)
 {
     int		from, to;
     int		result;
-    char	old, new;
+    char	oldc, newc;
     int		i;
     char	*str;
     int		last;
@@ -923,25 +973,24 @@ SFcheckFiles(dir)
     {
 	str = dir->entries[i].real;
 	last = strlen(str) - 1;
-	old = str[last];
+	oldc = str[last];
 	str[last] = 0;
-	if (stat(str, &statBuf))
-	    new = ' ';
+	if (mch_stat(str, &statBuf))
+	    newc = ' ';
 	else
-	    new = SFstatChar(&statBuf);
-	str[last] = new;
-	if (new != old)
+	    newc = SFstatChar(&statBuf);
+	str[last] = newc;
+	if (newc != oldc)
 	    result = 1;
     }
 
     return result;
 }
 
-/* ARGSUSED */
     static void
 SFdirModTimer(cl, id)
-    XtPointer		cl;
-    XtIntervalId	*id;
+    XtPointer		cl UNUSED;
+    XtIntervalId	*id UNUSED;
 {
     static int		n = -1;
     static int		f = 0;
@@ -1003,9 +1052,17 @@ SFstatChar(statBuf)
 
 /***************** Draw.c */
 
-#include <X11/Xaw/Cardinals.h>
+#ifdef FEAT_GUI_NEXTAW
+# include <X11/neXtaw/Cardinals.h>
+#else
+# include <X11/Xaw/Cardinals.h>
+#endif
 
-#define SF_DEFAULT_FONT "9x15"
+#ifdef FEAT_XFONTSET
+# define SF_DEFAULT_FONT "-misc-fixed-medium-r-normal--14-*"
+#else
+# define SF_DEFAULT_FONT "9x15"
+#endif
 
 #ifdef ABS
 # undef ABS
@@ -1015,17 +1072,26 @@ SFstatChar(statBuf)
 typedef struct
 {
     char *fontname;
-} TextData, *textPtr;
+} TextData;
 
 static GC SFlineGC, SFscrollGC, SFinvertGC, SFtextGC;
 
 static XtResource textResources[] =
 {
+#ifdef FEAT_XFONTSET
+	{XtNfontSet, XtCFontSet, XtRString, sizeof (char *),
+		XtOffsetOf(TextData, fontname), XtRString, SF_DEFAULT_FONT},
+#else
 	{XtNfont, XtCFont, XtRString, sizeof (char *),
-		XtOffset(textPtr, fontname), XtRString, SF_DEFAULT_FONT},
+		XtOffsetOf(TextData, fontname), XtRString, SF_DEFAULT_FONT},
+#endif
 };
 
+#ifdef FEAT_XFONTSET
+static XFontSet SFfont;
+#else
 static XFontStruct *SFfont;
+#endif
 
 static int SFcurrentListY;
 
@@ -1037,29 +1103,49 @@ static void SFinitFont __ARGS((void));
 SFinitFont()
 {
     TextData	*data;
+#ifdef FEAT_XFONTSET
+    XFontSetExtents *extents;
+    char **missing, *def_str;
+    int  num_missing;
+#endif
 
     data = XtNew(TextData);
 
     XtGetApplicationResources(selFileForm, (XtPointer) data, textResources,
 	    XtNumber(textResources), (Arg *) NULL, ZERO);
 
+#ifdef FEAT_XFONTSET
+    SFfont = XCreateFontSet(SFdisplay, data->fontname,
+			    &missing, &num_missing, &def_str);
+#else
     SFfont = XLoadQueryFont(SFdisplay, data->fontname);
+#endif
     if (!SFfont)
     {
+#ifdef FEAT_XFONTSET
+	SFfont = XCreateFontSet(SFdisplay, SF_DEFAULT_FONT,
+					    &missing, &num_missing, &def_str);
+#else
 	SFfont = XLoadQueryFont(SFdisplay, SF_DEFAULT_FONT);
+#endif
 	if (!SFfont)
 	{
-	    char	sbuf[256];
-
-	    (void)sprintf(sbuf, "vim_SelFile: can't get font %s",
-			   SF_DEFAULT_FONT);
-	    XtAppError(SFapp, sbuf);
+	    EMSG2(_("E616: vim_SelFile: can't get font %s"), SF_DEFAULT_FONT);
+	    SFstatus = SEL_FILE_CANCEL;
+	    return;
 	}
     }
 
+#ifdef FEAT_XFONTSET
+    extents = XExtentsOfFontSet(SFfont);
+    SFcharWidth = extents->max_logical_extent.width;
+    SFcharAscent = -extents->max_logical_extent.y;
+    SFcharHeight = extents->max_logical_extent.height;
+#else
     SFcharWidth = (SFfont->max_bounds.width + SFfont->min_bounds.width) / 2;
     SFcharAscent = SFfont->max_bounds.ascent;
     SFcharHeight = SFcharAscent + SFfont->max_bounds.descent;
+#endif
 }
 
 static void SFcreateGC __ARGS((void));
@@ -1093,12 +1179,18 @@ SFcreateGC()
 
     gcValues.foreground = SFfore;
     gcValues.background = SFback;
+#ifndef FEAT_XFONTSET
     gcValues.font = SFfont->fid;
+#endif
 
     SFtextGC = XCreateGC(
 	    SFdisplay,
 	    XtWindow(selFileLists[0]),
+#ifdef FEAT_XFONTSET
+	    (unsigned long)GCForeground | GCBackground,
+#else
 	    (unsigned long)GCForeground | GCBackground | GCFont,
+#endif
 	    &gcValues);
 
     rectangles[0].x = SFlineToTextH + SFbesideText;
@@ -1135,6 +1227,15 @@ SFclearList(n, doScroll)
 
 	if ((SFdirPtr + n < SFdirEnd) && dir->nEntries && dir->nChars)
 	{
+#ifdef FEAT_GUI_NEXTAW
+	    XawScrollbarSetThumb(
+		    selFileVScrolls[n],
+		    (float) (((double) dir->vOrigin) /
+			     dir->nEntries),
+		    (float) (((double) ((dir->nEntries < SFlistSize)
+					? dir->nEntries : SFlistSize)) /
+			     dir->nEntries));
+#else
 	    vim_XawScrollbarSetThumb(
 		    selFileVScrolls[n],
 		    (float) (((double) dir->vOrigin) /
@@ -1143,7 +1244,16 @@ SFclearList(n, doScroll)
 					? dir->nEntries : SFlistSize)) /
 			     dir->nEntries),
 		    (double)dir->nEntries);
+#endif
 
+#ifdef FEAT_GUI_NEXTAW
+	    XawScrollbarSetThumb(
+		    selFileHScrolls[n],
+		    (float) (((double) dir->hOrigin) / dir->nChars),
+		    (float) (((double) ((dir->nChars <
+					 SFcharsPerEntry) ? dir->nChars :
+					SFcharsPerEntry)) / dir->nChars));
+#else
 	    vim_XawScrollbarSetThumb(
 		    selFileHScrolls[n],
 		    (float) (((double) dir->hOrigin) / dir->nChars),
@@ -1151,13 +1261,24 @@ SFclearList(n, doScroll)
 					 SFcharsPerEntry) ? dir->nChars :
 					SFcharsPerEntry)) / dir->nChars),
 		    (double)dir->nChars);
+#endif
 	}
 	else
 	{
+#ifdef FEAT_GUI_NEXTAW
+	    XawScrollbarSetThumb(selFileVScrolls[n], (float) 0.0,
+		    (float) 1.0);
+#else
 	    vim_XawScrollbarSetThumb(selFileVScrolls[n], (float) 0.0,
 		    (float) 1.0, 1.0);
+#endif
+#ifdef FEAT_GUI_NEXTAW
+	    XawScrollbarSetThumb(selFileHScrolls[n], (float) 0.0,
+		    (float) 1.0);
+#else
 	    vim_XawScrollbarSetThumb(selFileHScrolls[n], (float) 0.0,
 		    (float) 1.0, 1.0);
+#endif
 	}
     }
 }
@@ -1169,10 +1290,10 @@ SFdeleteEntry(dir, entry)
     SFDir	*dir;
     SFEntry	*entry;
 {
-    register SFEntry	*e;
-    register SFEntry	*end;
-    int			n;
-    int			idx;
+    SFEntry	*e;
+    SFEntry	*end;
+    int		n;
+    int		idx;
 
     idx = entry - dir->entries;
 
@@ -1200,12 +1321,20 @@ SFdeleteEntry(dir, entry)
     if ((n < 0) || (n > 2))
 	return;
 
+#ifdef FEAT_GUI_NEXTAW
+    XawScrollbarSetThumb(
+	    selFileVScrolls[n],
+	    (float) (((double) dir->vOrigin) / dir->nEntries),
+	    (float) (((double) ((dir->nEntries < SFlistSize) ?
+				dir->nEntries : SFlistSize)) / dir->nEntries));
+#else
     vim_XawScrollbarSetThumb(
 	    selFileVScrolls[n],
 	    (float) (((double) dir->vOrigin) / dir->nEntries),
 	    (float) (((double) ((dir->nEntries < SFlistSize) ?
 				dir->nEntries : SFlistSize)) / dir->nEntries),
 	    (double)dir->nEntries);
+#endif
 }
 
 static void SFwriteStatChar __ARGS((char *name, int last, struct stat *statBuf));
@@ -1241,10 +1370,10 @@ SFstatAndCheck(dir, entry)
 	last = strlen(entry->real) - 1;
 	entry->real[last] = 0;
 	entry->statDone = 1;
-	if ((!stat(entry->real, &statBuf))
+	if ((!mch_stat(entry->real, &statBuf))
 #ifdef S_IFLNK
-		|| (!lstat(entry->real, &statBuf))
-#endif /* ndef S_IFLNK */
+		|| (!mch_lstat(entry->real, &statBuf))
+#endif
 	   )
 	{
 	    if (SFfunc)
@@ -1286,14 +1415,14 @@ SFstatAndCheck(dir, entry)
 
     static void
 SFdrawStrings(w, dir, from, to)
-    register Window	w;
-    register SFDir	*dir;
-    register int	from;
-    register int	to;
+    Window	w;
+    SFDir	*dir;
+    int		from;
+    int		to;
 {
-    register int	i;
-    register SFEntry	*entry;
-    int			x;
+    int		i;
+    SFEntry	*entry;
+    int		x;
 
     x = SFtextX - dir->hOrigin * SFcharWidth;
 
@@ -1312,6 +1441,17 @@ SFdrawStrings(w, dir, from, to)
 		continue;
 	    }
 	}
+#ifdef FEAT_XFONTSET
+	XmbDrawImageString(
+		SFdisplay,
+		w,
+		SFfont,
+		SFtextGC,
+		x,
+		SFtextYoffset + i * SFentryHeight,
+		entry->shown,
+		strlen(entry->shown));
+#else
 	XDrawImageString(
 		SFdisplay,
 		w,
@@ -1320,6 +1460,7 @@ SFdrawStrings(w, dir, from, to)
 		SFtextYoffset + i * SFentryHeight,
 		entry->shown,
 		strlen(entry->shown));
+#endif
 	if (dir->vOrigin + i == dir->beginSelection)
 	{
 	    XDrawLine(
@@ -1373,6 +1514,17 @@ SFdrawList(n, doScroll)
     {
 	dir = &(SFdirs[SFdirPtr + n]);
 	w = XtWindow(selFileLists[n]);
+#ifdef FEAT_XFONTSET
+	XmbDrawImageString(
+		SFdisplay,
+		w,
+		SFfont,
+		SFtextGC,
+		SFtextX - dir->hOrigin * SFcharWidth,
+		SFlineToTextV + SFaboveAndBelowText + SFcharAscent,
+		dir->dir,
+		strlen(dir->dir));
+#else
 	XDrawImageString(
 		SFdisplay,
 		w,
@@ -1381,6 +1533,7 @@ SFdrawList(n, doScroll)
 		SFlineToTextV + SFaboveAndBelowText + SFcharAscent,
 		dir->dir,
 		strlen(dir->dir));
+#endif
 	SFdrawStrings(w, dir, 0, SFlistSize - 1);
     }
 }
@@ -1397,7 +1550,7 @@ SFdrawLists(doScroll)
 
     static void
 SFinvertEntry(n)
-    register int	n;
+    int		n;
 {
     XFillRectangle(
 	    SFdisplay,
@@ -1440,11 +1593,10 @@ SFscrollTimerInterval()
 
 static void SFscrollTimer __ARGS((XtPointer p, XtIntervalId *id));
 
-/* ARGSUSED */
     static void
 SFscrollTimer(p, id)
     XtPointer		p;
-    XtIntervalId	*id;
+    XtIntervalId	*id UNUSED;
 {
     SFDir	*dir;
     int		save;
@@ -1470,27 +1622,36 @@ SFscrollTimer(p, id)
     {
 	if (dir->nEntries)
 	{
+#ifdef FEAT_GUI_NEXTAW
+	    XawScrollbarSetThumb(
+		    selFileVScrolls[n],
+		    (float) (((double) dir->vOrigin) / dir->nEntries),
+		    (float) (((double) ((dir->nEntries < SFlistSize) ?
+				dir->nEntries : SFlistSize)) / dir->nEntries));
+#else
 	    vim_XawScrollbarSetThumb(
 		    selFileVScrolls[n],
 		    (float) (((double) dir->vOrigin) / dir->nEntries),
 		    (float) (((double) ((dir->nEntries < SFlistSize) ?
 				dir->nEntries : SFlistSize)) / dir->nEntries),
 		    (double)dir->nEntries);
+#endif
 	}
     }
 
     if (SFbuttonPressed)
 	SFscrollTimerId = XtAppAddTimeOut(SFapp,
-		       SFscrollTimerInterval(), SFscrollTimer, (XtPointer) n);
+		       SFscrollTimerInterval(), SFscrollTimer,
+		       (XtPointer)(long_u)n);
 }
 
     static int
 SFnewInvertEntry(n, event)
-    register int		n;
-    register XMotionEvent	*event;
+    int			n;
+    XMotionEvent	*event;
 {
-    register int	x, y;
-    register int	new;
+    int			x, y;
+    int			nw;
     static int		SFscrollTimerAdded = 0;
 
     x = event->x;
@@ -1501,7 +1662,7 @@ SFnewInvertEntry(n, event)
 
     if ((x >= 0) && (x <= SFupperX) && (y >= SFlowerY) && (y <= SFupperY))
     {
-	register SFDir *dir = &(SFdirs[SFdirPtr + n]);
+	SFDir *dir = &(SFdirs[SFdirPtr + n]);
 
 	if (SFscrollTimerAdded)
 	{
@@ -1509,10 +1670,10 @@ SFnewInvertEntry(n, event)
 	    XtRemoveTimeOut(SFscrollTimerId);
 	}
 
-	new = (y - SFlowerY) / SFentryHeight;
-	if (dir->vOrigin + new >= dir->nEntries)
+	nw = (y - SFlowerY) / SFentryHeight;
+	if (dir->vOrigin + nw >= dir->nEntries)
 	    return -1;
-	return new;
+	return nw;
     }
     else
     {
@@ -1524,21 +1685,20 @@ SFnewInvertEntry(n, event)
 		SFscrollTimerAdded = 1;
 		SFscrollTimerId = XtAppAddTimeOut(SFapp,
 			SFscrollTimerInterval(), SFscrollTimer,
-			(XtPointer) n);
+			(XtPointer)(long_u)n);
 	    }
 	}
 	return -1;
     }
 }
 
-/* ARGSUSED */
     static void
 SFenterList(w, n, event)
-    Widget				w;
-    register int			n;
-    register XEnterWindowEvent		*event;
+    Widget		w UNUSED;
+    int			n;
+    XEnterWindowEvent	*event;
 {
-    register int	new;
+    int			nw;
 
     /* sanity */
     if (SFcurrentInvert[n] != -1)
@@ -1547,20 +1707,19 @@ SFenterList(w, n, event)
 	SFcurrentInvert[n] = -1;
     }
 
-    new = SFnewInvertEntry(n, (XMotionEvent *) event);
-    if (new != -1)
+    nw = SFnewInvertEntry(n, (XMotionEvent *) event);
+    if (nw != -1)
     {
-	SFcurrentInvert[n] = new;
+	SFcurrentInvert[n] = nw;
 	SFinvertEntry(n);
     }
 }
 
-/* ARGSUSED */
     static void
 SFleaveList(w, n, event)
-    Widget		w;
-    register int	n;
-    XEvent		*event;
+    Widget	w UNUSED;
+    int		n;
+    XEvent	*event UNUSED;
 {
     if (SFcurrentInvert[n] != -1)
     {
@@ -1569,64 +1728,61 @@ SFleaveList(w, n, event)
     }
 }
 
-/* ARGSUSED */
     static void
 SFmotionList(w, n, event)
-    Widget			w;
-    register int		n;
-    register XMotionEvent	*event;
+    Widget		w UNUSED;
+    int			n;
+    XMotionEvent	*event;
 {
-    register int	new;
+    int		nw;
 
-    new = SFnewInvertEntry(n, event);
+    nw = SFnewInvertEntry(n, event);
 
-    if (new != SFcurrentInvert[n])
+    if (nw != SFcurrentInvert[n])
     {
 	if (SFcurrentInvert[n] != -1)
 	    SFinvertEntry(n);
-	SFcurrentInvert[n] = new;
-	if (new != -1)
+	SFcurrentInvert[n] = nw;
+	if (nw != -1)
 	    SFinvertEntry(n);
     }
 }
 
-/* ARGSUSED */
     static void
 SFvFloatSliderMovedCallback(w, n, fnew)
-    Widget		w;
+    Widget	w;
     XtPointer	n;
     XtPointer	fnew;
 {
-    int	new;
+    int		nw;
 
-    new = (*(float *)fnew) * SFdirs[SFdirPtr + (int)(long)n].nEntries;
-    SFvSliderMovedCallback(w, (int)(long)n, new);
+    nw = (*(float *)fnew) * SFdirs[SFdirPtr + (int)(long)n].nEntries;
+    SFvSliderMovedCallback(w, (int)(long)n, nw);
 }
 
-/* ARGSUSED */
     static void
-SFvSliderMovedCallback(w, n, new)
-    Widget	w;
-    int	n;
-    int	new;
+SFvSliderMovedCallback(w, n, nw)
+    Widget	w UNUSED;
+    int		n;
+    int		nw;
 {
     int		old;
-    register Window	win;
-    SFDir		*dir;
+    Window	win;
+    SFDir	*dir;
 
     dir = &(SFdirs[SFdirPtr + n]);
 
     old = dir->vOrigin;
-    dir->vOrigin = new;
+    dir->vOrigin = nw;
 
-    if (old == new)
+    if (old == nw)
 	return;
 
     win = XtWindow(selFileLists[n]);
 
-    if (ABS(new - old) < SFlistSize)
+    if (ABS(nw - old) < SFlistSize)
     {
-	if (new > old)
+	if (nw > old)
 	{
 	    XCopyArea(
 		    SFdisplay,
@@ -1634,21 +1790,21 @@ SFvSliderMovedCallback(w, n, new)
 		    win,
 		    SFscrollGC,
 		    SFlineToTextH,
-		    SFlowerY + (new - old) * SFentryHeight,
+		    SFlowerY + (nw - old) * SFentryHeight,
 		    SFentryWidth + SFlineToTextH,
-		    (SFlistSize - (new - old)) * SFentryHeight,
+		    (SFlistSize - (nw - old)) * SFentryHeight,
 		    SFlineToTextH,
 		    SFlowerY);
 	    XClearArea(
 		    SFdisplay,
 		    win,
 		    SFlineToTextH,
-		    SFlowerY + (SFlistSize - (new - old)) *
+		    SFlowerY + (SFlistSize - (nw - old)) *
 		    SFentryHeight,
 		    SFentryWidth + SFlineToTextH,
-		    (new - old) * SFentryHeight,
+		    (nw - old) * SFentryHeight,
 		    False);
-	    SFdrawStrings(win, dir, SFlistSize - (new - old),
+	    SFdrawStrings(win, dir, SFlistSize - (nw - old),
 		    SFlistSize - 1);
 	}
 	else
@@ -1661,18 +1817,18 @@ SFvSliderMovedCallback(w, n, new)
 		    SFlineToTextH,
 		    SFlowerY,
 		    SFentryWidth + SFlineToTextH,
-		    (SFlistSize - (old - new)) * SFentryHeight,
+		    (SFlistSize - (old - nw)) * SFentryHeight,
 		    SFlineToTextH,
-		    SFlowerY + (old - new) * SFentryHeight);
+		    SFlowerY + (old - nw) * SFentryHeight);
 	    XClearArea(
 		    SFdisplay,
 		    win,
 		    SFlineToTextH,
 		    SFlowerY,
 		    SFentryWidth + SFlineToTextH,
-		    (old - new) * SFentryHeight,
+		    (old - nw) * SFentryHeight,
 		    False);
-	    SFdrawStrings(win, dir, 0, old - new);
+	    SFdrawStrings(win, dir, 0, old - nw);
 	}
     }
     else
@@ -1689,105 +1845,149 @@ SFvSliderMovedCallback(w, n, new)
     }
 }
 
-/* ARGSUSED */
     static void
 SFvAreaSelectedCallback(w, n, pnew)
-    Widget		w;
+    Widget	w;
     XtPointer	n;
     XtPointer	pnew;
 {
     SFDir	*dir;
-    int	new;
+    int		nw = (int)(long)pnew;
 
     dir = &(SFdirs[SFdirPtr + (int)(long)n]);
 
-    new = dir->vOrigin + (int)(long)pnew;
+#ifdef FEAT_GUI_NEXTAW
+    if (nw < 0)
+    {
+	if (nw > -SFvScrollHeight)
+	    nw = -1;
+	else
+	    nw = -SFlistSize;
+    }
+    else if (nw > 0)
+    {
+	if (nw < SFvScrollHeight)
+	    nw = 1;
+	else
+	    nw = SFlistSize;
+    }
+#endif
+    nw += dir->vOrigin;
 
-    if (new > dir->nEntries - SFlistSize)
-	new = dir->nEntries - SFlistSize;
+    if (nw > dir->nEntries - SFlistSize)
+	nw = dir->nEntries - SFlistSize;
 
-    if (new < 0)
-	new = 0;
+    if (nw < 0)
+	nw = 0;
 
     if (dir->nEntries)
     {
 	float	f;
 
-	f = ((double) new) / dir->nEntries;
+	f = ((double) nw) / dir->nEntries;
 
+#ifdef FEAT_GUI_NEXTAW
+	XawScrollbarSetThumb(
+		w,
+		f,
+		(float) (((double) ((dir->nEntries < SFlistSize) ?
+				dir->nEntries : SFlistSize)) / dir->nEntries));
+#else
 	vim_XawScrollbarSetThumb(
 		w,
 		f,
 		(float) (((double) ((dir->nEntries < SFlistSize) ?
 				dir->nEntries : SFlistSize)) / dir->nEntries),
 		(double)dir->nEntries);
+#endif
     }
 
-    SFvSliderMovedCallback(w, (int)(long)n, new);
+    SFvSliderMovedCallback(w, (int)(long)n, nw);
 }
 
-/* ARGSUSED */
     static void
-SFhSliderMovedCallback(w, n, new)
-    Widget	w;
+SFhSliderMovedCallback(w, n, nw)
+    Widget	w UNUSED;
     XtPointer	n;
-    XtPointer	new;
+    XtPointer	nw;
 {
     SFDir	*dir;
     int	save;
 
     dir = &(SFdirs[SFdirPtr + (int)(long)n]);
     save = dir->hOrigin;
-    dir->hOrigin = (*(float *)new) * dir->nChars;
+    dir->hOrigin = (*(float *)nw) * dir->nChars;
     if (dir->hOrigin == save)
 	return;
 
     SFdrawList((int)(long)n, SF_DO_NOT_SCROLL);
 }
 
-/* ARGSUSED */
     static void
 SFhAreaSelectedCallback(w, n, pnew)
-    Widget		w;
+    Widget	w;
     XtPointer	n;
     XtPointer	pnew;
 {
     SFDir	*dir;
-    int	new;
+    int		nw = (int)(long)pnew;
 
     dir = &(SFdirs[SFdirPtr + (int)(long)n]);
 
-    new = dir->hOrigin + (int)(long)pnew;
+#ifdef FEAT_GUI_NEXTAW
+    if (nw < 0)
+    {
+	if (nw > -SFhScrollWidth)
+	    nw = -1;
+	else
+	    nw = -SFcharsPerEntry;
+    }
+    else if (nw > 0)
+    {
+	if (nw < SFhScrollWidth)
+	    nw = 1;
+	else
+	    nw = SFcharsPerEntry;
+    }
+#endif
+    nw += dir->hOrigin;
 
-    if (new > dir->nChars - SFcharsPerEntry)
-	new = dir->nChars - SFcharsPerEntry;
+    if (nw > dir->nChars - SFcharsPerEntry)
+	nw = dir->nChars - SFcharsPerEntry;
 
-    if (new < 0)
-	new = 0;
+    if (nw < 0)
+	nw = 0;
 
     if (dir->nChars)
     {
 	float	f;
 
-	f = ((double) new) / dir->nChars;
+	f = ((double) nw) / dir->nChars;
 
+#ifdef FEAT_GUI_NEXTAW
+	XawScrollbarSetThumb(
+		w,
+		f,
+		(float) (((double) ((dir->nChars < SFcharsPerEntry) ?
+			       dir->nChars : SFcharsPerEntry)) / dir->nChars));
+#else
 	vim_XawScrollbarSetThumb(
 		w,
 		f,
 		(float) (((double) ((dir->nChars < SFcharsPerEntry) ?
 			       dir->nChars : SFcharsPerEntry)) / dir->nChars),
 		(double)dir->nChars);
+#endif
 
 	SFhSliderMovedCallback(w, n, (XtPointer)&f);
     }
 }
 
-/* ARGSUSED */
     static void
-SFpathSliderMovedCallback(w, client_data, new)
-    Widget		w;
-    XtPointer	client_data;
-    XtPointer	new;
+SFpathSliderMovedCallback(w, client_data, nw)
+    Widget	w UNUSED;
+    XtPointer	client_data UNUSED;
+    XtPointer	nw;
 {
     SFDir		*dir;
     int			n;
@@ -1795,7 +1995,7 @@ SFpathSliderMovedCallback(w, client_data, new)
     int			SFdirPtrSave;
 
     SFdirPtrSave = SFdirPtr;
-    SFdirPtr = (*(float *)new) * SFdirEnd;
+    SFdirPtr = (*(float *)nw) * SFdirEnd;
     if (SFdirPtr == SFdirPtrSave)
 	return;
 
@@ -1819,31 +2019,53 @@ SFpathSliderMovedCallback(w, client_data, new)
     XawTextSetInsertionPoint(selFileField, pos);
 }
 
-/* ARGSUSED */
     static void
 SFpathAreaSelectedCallback(w, client_data, pnew)
     Widget	w;
-    XtPointer	client_data;
+    XtPointer	client_data UNUSED;
     XtPointer	pnew;
 {
-    int	new;
+    int		nw = (int)(long)pnew;
     float	f;
 
-    new = SFdirPtr + (int)(long)pnew;
+#ifdef FEAT_GUI_NEXTAW
+    if (nw < 0)
+    {
+	if (nw > -SFpathScrollWidth)
+	    nw = -1;
+	else
+	    nw = -3;
+    }
+    else if (nw > 0)
+    {
+	if (nw < SFpathScrollWidth)
+	    nw = 1;
+	else
+	    nw = 3;
+    }
+#endif
+    nw += SFdirPtr;
 
-    if (new > SFdirEnd - 3)
-	new = SFdirEnd - 3;
+    if (nw > SFdirEnd - 3)
+	nw = SFdirEnd - 3;
 
-    if (new < 0)
-	new = 0;
+    if (nw < 0)
+	nw = 0;
 
-    f = ((double) new) / SFdirEnd;
+    f = ((double) nw) / SFdirEnd;
 
+#ifdef FEAT_GUI_NEXTAW
+    XawScrollbarSetThumb(
+	    w,
+	    f,
+	    (float) (((double) ((SFdirEnd < 3) ? SFdirEnd : 3)) / SFdirEnd));
+#else
     vim_XawScrollbarSetThumb(
 	    w,
 	    f,
 	    (float) (((double) ((SFdirEnd < 3) ? SFdirEnd : 3)) / SFdirEnd),
 	    (double)SFdirEnd);
+#endif
 
     SFpathSliderMovedCallback(w, (XtPointer) NULL, (XtPointer)&f);
 }
@@ -1851,8 +2073,8 @@ SFpathAreaSelectedCallback(w, client_data, pnew)
     static Boolean
 SFworkProc()
 {
-    register SFDir	*dir;
-    register SFEntry	*entry;
+    SFDir	*dir;
+    SFEntry	*entry;
 
     for (dir = &(SFdirs[SFdirEnd - 1]); dir >= SFdirs; dir--)
     {
@@ -1912,7 +2134,7 @@ SFgetDir(dir)
     if (!dirp)
 	return 1;
 
-    (void) stat(".", &statBuf);
+    (void)mch_stat(".", &statBuf);
     dir->mtime = statBuf.st_mtime;
 
     while ((dp = readdir(dirp)))
@@ -1954,11 +2176,15 @@ SFgetDir(dir)
 #include <X11/cursorfont.h>
 #include <X11/Composite.h>
 #include <X11/Shell.h>
+#ifdef FEAT_GUI_NEXTAW
+# include <X11/neXtaw/Form.h>
+# include <X11/neXtaw/Command.h>
+# include <X11/neXtaw/Label.h>
+#else
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Command.h>
 #include <X11/Xaw/Label.h>
-
-static int SFstatus = SEL_FILE_NULL;
+#endif
 
 static char *oneLineTextEditTranslations = "\
 	<Key>Return:	redraw-display()\n\
@@ -1967,13 +2193,12 @@ static char *oneLineTextEditTranslations = "\
 
 static void SFexposeList __ARGS((Widget w, XtPointer n, XEvent *event, Boolean *cont));
 
-/* ARGSUSED */
     static void
 SFexposeList(w, n, event, cont)
-    Widget	w;
+    Widget	w UNUSED;
     XtPointer	n;
     XEvent	*event;
-    Boolean	*cont;
+    Boolean	*cont UNUSED;
 {
     if ((event->type == NoExpose) || event->xexpose.count)
 	return;
@@ -1983,13 +2208,12 @@ SFexposeList(w, n, event, cont)
 
 static void SFmodVerifyCallback __ARGS((Widget w, XtPointer client_data, XEvent *event, Boolean *cont));
 
-/* ARGSUSED */
     static void
 SFmodVerifyCallback(w, client_data, event, cont)
-    Widget		w;
-    XtPointer		client_data;
+    Widget		w UNUSED;
+    XtPointer		client_data UNUSED;
     XEvent		*event;
-    Boolean		*cont;
+    Boolean		*cont UNUSED;
 {
     char	buf[2];
 
@@ -2002,11 +2226,11 @@ SFmodVerifyCallback(w, client_data, event, cont)
 
 static void SFokCallback __ARGS((Widget w, XtPointer cl, XtPointer cd));
 
-/* ARGSUSED */
     static void
 SFokCallback(w, cl, cd)
-    Widget	w;
-    XtPointer	cl, cd;
+    Widget	w UNUSED;
+    XtPointer	cl UNUSED;
+    XtPointer	cd UNUSED;
 {
     SFstatus = SEL_FILE_OK;
 }
@@ -2019,11 +2243,11 @@ static XtCallbackRec SFokSelect[] =
 
 static void SFcancelCallback __ARGS((Widget w, XtPointer cl, XtPointer cd));
 
-/* ARGSUSED */
     static void
 SFcancelCallback(w, cl, cd)
-    Widget	w;
-    XtPointer	cl, cd;
+    Widget	w UNUSED;
+    XtPointer	cl UNUSED;
+    XtPointer	cd UNUSED;
 {
     SFstatus = SEL_FILE_CANCEL;
 }
@@ -2036,16 +2260,15 @@ static XtCallbackRec SFcancelSelect[] =
 
 static void SFdismissAction __ARGS((Widget w, XEvent *event, String *params, Cardinal *num_params));
 
-/* ARGSUSED */
     static void
 SFdismissAction(w, event, params, num_params)
-    Widget	w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+    Widget	w UNUSED;
+    XEvent	*event;
+    String	*params UNUSED;
+    Cardinal	*num_params UNUSED;
 {
-    if (event->type == ClientMessage &&
-	    event->xclient.data.l[0] != SFwmDeleteWindow)
+    if (event->type == ClientMessage
+	    && (Atom)event->xclient.data.l[0] != SFwmDeleteWindow)
 	return;
 
     SFstatus = SEL_FILE_CANCEL;
@@ -2060,6 +2283,88 @@ static XtActionsRec actions[] =
     {"SelFileDismiss",	SFdismissAction},
 };
 
+    static void
+SFsetColors(bg, fg, scroll_bg, scroll_fg)
+    guicolor_T	bg;
+    guicolor_T	fg;
+    guicolor_T	scroll_bg;
+    guicolor_T	scroll_fg;
+{
+    if (selFileForm)
+    {
+	XtVaSetValues(selFileForm, XtNbackground,  bg,
+				   XtNforeground,  fg,
+				   XtNborderColor, bg,
+				   NULL);
+    }
+    {
+	int i;
+
+	for (i = 0; i < 3; ++i)
+	{
+	    if (selFileLists[i])
+	    {
+		XtVaSetValues(selFileLists[i], XtNbackground,  bg,
+					       XtNforeground,  fg,
+					       XtNborderColor, fg,
+					       NULL);
+	    }
+	}
+    }
+    if (selFileOK)
+    {
+	XtVaSetValues(selFileOK, XtNbackground,  bg,
+				 XtNforeground,  fg,
+				 XtNborderColor, fg,
+				 NULL);
+    }
+    if (selFileCancel)
+    {
+	XtVaSetValues(selFileCancel, XtNbackground, bg,
+				     XtNforeground, fg,
+				     XtNborderColor, fg,
+				     NULL);
+    }
+    if (selFilePrompt)
+    {
+	XtVaSetValues(selFilePrompt, XtNbackground, bg,
+				     XtNforeground, fg,
+				     NULL);
+    }
+    if (gui.dpy)
+    {
+	XSetBackground(gui.dpy, SFtextGC, bg);
+	XSetForeground(gui.dpy, SFtextGC, fg);
+	XSetForeground(gui.dpy, SFlineGC, fg);
+
+	/* This is an xor GC, so combine the fg and background */
+	XSetBackground(gui.dpy, SFinvertGC, fg ^ bg);
+	XSetForeground(gui.dpy, SFinvertGC, fg ^ bg);
+    }
+    if (selFileHScroll)
+    {
+	XtVaSetValues(selFileHScroll, XtNbackground, scroll_bg,
+				      XtNforeground, scroll_fg,
+				      XtNborderColor, fg,
+				      NULL);
+    }
+    {
+	int i;
+
+	for (i = 0; i < 3; i++)
+	{
+	    XtVaSetValues(selFileVScrolls[i], XtNbackground, scroll_bg,
+					      XtNforeground, scroll_fg,
+					      XtNborderColor, fg,
+					      NULL);
+	    XtVaSetValues(selFileHScrolls[i], XtNbackground, scroll_bg,
+					      XtNforeground, scroll_fg,
+					      XtNborderColor, fg,
+					      NULL);
+	}
+    }
+}
+
     static void
 SFcreateWidgets(toplevel, prompt, ok, cancel)
     Widget	toplevel;
@@ -2077,6 +2382,7 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
     selFile = XtVaAppCreateShell("selFile", "SelFile",
 		transientShellWidgetClass, SFdisplay,
 		XtNtransientFor, toplevel,
+		XtNtitle, prompt,
 		NULL);
 
     /* Add WM_DELETE_WINDOW protocol */
@@ -2185,7 +2491,11 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
     XtSetKeyboardFocus(selFileForm, selFileField);
 
     selFileHScroll = XtVaCreateManagedWidget("selFileHScroll",
+#ifdef FEAT_GUI_NEXTAW
+		scrollbarWidgetClass, selFileForm,
+#else
 		vim_scrollbarWidgetClass, selFileForm,
+#endif
 		XtNorientation, XtorientHorizontal,
 		XtNwidth, SFpathScrollWidth,
 		XtNheight, scrollThickness,
@@ -2198,7 +2508,9 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
 		XtNright, XtChainLeft,
 		XtNforeground, gui.scroll_fg_pixel,
 		XtNbackground, gui.scroll_bg_pixel,
+#ifndef FEAT_GUI_NEXTAW
 		XtNlimitThumb, 1,
+#endif
 		NULL);
 
     XtAddCallback(selFileHScroll, XtNjumpProc,
@@ -2258,7 +2570,11 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
     for (n = 0; n < 3; n++)
     {
 	selFileVScrolls[n] = XtVaCreateManagedWidget("selFileVScroll",
+#ifdef FEAT_GUI_NEXTAW
+		    scrollbarWidgetClass, selFileLists[n],
+#else
 		    vim_scrollbarWidgetClass, selFileLists[n],
+#endif
 		    XtNx, vScrollX,
 		    XtNy, vScrollY,
 		    XtNwidth, scrollThickness,
@@ -2266,16 +2582,23 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
 		    XtNborderColor, SFfore,
 		    XtNforeground, gui.scroll_fg_pixel,
 		    XtNbackground, gui.scroll_bg_pixel,
+#ifndef FEAT_GUI_NEXTAW
 		    XtNlimitThumb, 1,
+#endif
 		    NULL);
 
 	XtAddCallback(selFileVScrolls[n], XtNjumpProc,
-		(XtCallbackProc)SFvFloatSliderMovedCallback, (XtPointer)n);
+		(XtCallbackProc)SFvFloatSliderMovedCallback,
+		(XtPointer)(long_u)n);
 	XtAddCallback(selFileVScrolls[n], XtNscrollProc,
-		(XtCallbackProc)SFvAreaSelectedCallback, (XtPointer)n);
+		(XtCallbackProc)SFvAreaSelectedCallback, (XtPointer)(long_u)n);
 
 	selFileHScrolls[n] = XtVaCreateManagedWidget("selFileHScroll",
+#ifdef FEAT_GUI_NEXTAW
+		    scrollbarWidgetClass, selFileLists[n],
+#else
 		    vim_scrollbarWidgetClass, selFileLists[n],
+#endif
 		    XtNorientation, XtorientHorizontal,
 		    XtNx, hScrollX,
 		    XtNy, hScrollY,
@@ -2284,13 +2607,16 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
 		    XtNborderColor, SFfore,
 		    XtNforeground, gui.scroll_fg_pixel,
 		    XtNbackground, gui.scroll_bg_pixel,
+#ifndef FEAT_GUI_NEXTAW
 		    XtNlimitThumb, 1,
+#endif
 		    NULL);
 
 	XtAddCallback(selFileHScrolls[n], XtNjumpProc,
-		(XtCallbackProc)SFhSliderMovedCallback, (XtPointer)n);
+		(XtCallbackProc)SFhSliderMovedCallback,
+		(XtPointer)(long_u)n);
 	XtAddCallback(selFileHScrolls[n], XtNscrollProc,
-		(XtCallbackProc)SFhAreaSelectedCallback, (XtPointer)n);
+		(XtCallbackProc)SFhAreaSelectedCallback, (XtPointer)(long_u)n);
     }
 
     selFileOK = XtVaCreateManagedWidget("selFileOK",
@@ -2310,7 +2636,7 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
 		XtNright, XtChainLeft,
 		NULL);
 
-    (void)XtVaCreateManagedWidget("selFileCancel",
+    selFileCancel = XtVaCreateManagedWidget("selFileCancel",
 		commandWidgetClass, selFileForm,
 		XtNlabel, cancel,
 		XtNresizable, True,
@@ -2340,17 +2666,17 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
     for (n = 0; n < 3; n++)
     {
 	XtAddEventHandler(selFileLists[n], ExposureMask, True,
-		(XtEventHandler)SFexposeList, (XtPointer)n);
+		(XtEventHandler)SFexposeList, (XtPointer)(long_u)n);
 	XtAddEventHandler(selFileLists[n], EnterWindowMask, False,
-		(XtEventHandler)SFenterList, (XtPointer)n);
+		(XtEventHandler)SFenterList, (XtPointer)(long_u)n);
 	XtAddEventHandler(selFileLists[n], LeaveWindowMask, False,
-		(XtEventHandler)SFleaveList, (XtPointer)n);
+		(XtEventHandler)SFleaveList, (XtPointer)(long_u)n);
 	XtAddEventHandler(selFileLists[n], PointerMotionMask, False,
-		(XtEventHandler)SFmotionList, (XtPointer)n);
+		(XtEventHandler)SFmotionList, (XtPointer)(long_u)n);
 	XtAddEventHandler(selFileLists[n], ButtonPressMask, False,
-		(XtEventHandler)SFbuttonPressList, (XtPointer)n);
+		(XtEventHandler)SFbuttonPressList, (XtPointer)(long_u)n);
 	XtAddEventHandler(selFileLists[n], ButtonReleaseMask, False,
-		(XtEventHandler)SFbuttonReleaseList, (XtPointer)n);
+		(XtEventHandler)SFbuttonReleaseList, (XtPointer)(long_u)n);
     }
 
     XtAddEventHandler(selFileField, KeyPressMask, False,
@@ -2362,7 +2688,26 @@ SFcreateWidgets(toplevel, prompt, ok, cancel)
     static void
 SFtextChanged()
 {
+#if defined(FEAT_XFONTSET) && defined(XtNinternational)
+    if ((unsigned long)_XawTextFormat((TextWidget)selFileField) == XawFmtWide)
+    {
+	wchar_t *wcbuf=(wchar_t *)SFtextBuffer;
 
+	if ((wcbuf[0] == L'/') || (wcbuf[0] == L'~'))
+	{
+	    (void) wcstombs(SFcurrentPath, wcbuf, MAXPATHL);
+	    SFtextPos = XawTextGetInsertionPoint(selFileField);
+	}
+	else
+	{
+	    strcpy(SFcurrentPath, SFstartDir);
+	    (void) wcstombs(SFcurrentPath + strlen(SFcurrentPath), wcbuf, MAXPATHL);
+
+	    SFtextPos = XawTextGetInsertionPoint(selFileField) + strlen(SFstartDir);
+	}
+    }
+    else
+#endif
     if ((SFtextBuffer[0] == '/') || (SFtextBuffer[0] == '~'))
     {
 	(void) strcpy(SFcurrentPath, SFtextBuffer);
@@ -2387,6 +2732,26 @@ SFtextChanged()
     static char *
 SFgetText()
 {
+#if defined(FEAT_XFONTSET) && defined(XtNinternational)
+    char *buf;
+
+    if ((unsigned long)_XawTextFormat((TextWidget)selFileField) == XawFmtWide)
+    {
+	wchar_t *wcbuf;
+	int mbslength;
+
+	XtVaGetValues(selFileField,
+	    XtNstring, &wcbuf,
+	NULL);
+	mbslength = wcstombs(NULL, wcbuf, 0);
+	/* Hack: some broken wcstombs() returns zero, just get a large buffer */
+	if (mbslength == 0 && wcbuf != NULL && wcbuf[0] != 0)
+	    mbslength = MAXPATHL;
+	buf=(char *)XtMalloc(mbslength + 1);
+	wcstombs(buf, wcbuf, mbslength +1);
+	return buf;
+    }
+#endif
     return (char *)vim_strsave((char_u *)SFtextBuffer);
 }
 
@@ -2398,41 +2763,53 @@ SFprepareToReturn()
     XtUnmapWidget(selFile);
     XtRemoveTimeOut(SFdirModTimerId);
     if (SFchdir(SFstartDir))
-	XtAppError(SFapp, "vim_SelFile: can't return to current directory");
+    {
+	EMSG(_("E614: vim_SelFile: can't return to current directory"));
+	SFstatus = SEL_FILE_CANCEL;
+    }
 }
 
     char *
-vim_SelFile(toplevel, prompt, init_path, show_entry, x, y, fg, bg)
+vim_SelFile(toplevel, prompt, init_path, show_entry, x, y, fg, bg, scroll_fg, scroll_bg)
     Widget	toplevel;
     char	*prompt;
     char	*init_path;
     int		(*show_entry)();
     int		x, y;
-    GuiColor	fg, bg;
+    guicolor_T	fg, bg;
+    guicolor_T	scroll_fg, scroll_bg; /* The "Scrollbar" group colors */
 {
     static int	firstTime = 1;
     XEvent	event;
     char	*name_return;
 
     if (prompt == NULL)
-	prompt = "Pathname:";
+	prompt = _("Pathname:");
     SFfore = fg;
     SFback = bg;
 
+    if (mch_dirname((char_u *)SFstartDir, MAXPATHL) == FAIL)
+    {
+	EMSG(_("E615: vim_SelFile: can't get current directory"));
+	return NULL;
+    }
+
     if (firstTime)
     {
 	firstTime = 0;
 	SFdisplay = XtDisplay(toplevel);
-	SFcreateWidgets(toplevel, prompt, "OK", "Cancel");
+	SFcreateWidgets(toplevel, prompt, _("OK"), _("Cancel"));
     }
     else
+    {
 	XtVaSetValues(selFilePrompt, XtNlabel, prompt, NULL);
+	XtVaSetValues(selFile, XtNtitle, prompt, NULL);
+	SFsetColors(bg, fg, scroll_bg, scroll_fg);
+    }
 
     XtVaSetValues(selFile, XtNx, x, XtNy, y, NULL);
     XtMapWidget(selFile);
 
-    if (mch_dirname((char_u *)SFstartDir, MAXPATHL) == FAIL)
-	XtAppError(SFapp, "vim_SelFile: can't get current directory");
     (void)strcat(SFstartDir, "/");
     (void)strcpy(SFcurrentDir, SFstartDir);
 
@@ -2464,7 +2841,7 @@ vim_SelFile(toplevel, prompt, init_path, show_entry, x, y, fg, bg)
     SFdirModTimerId = XtAppAddTimeOut(SFapp, (unsigned long) 1000,
 	    SFdirModTimer, (XtPointer) NULL);
 
-    while (1)
+    for (;;)
     {
 	XtAppNextEvent(SFapp, &event);
 	XtDispatchEvent(&event);
@@ -2486,5 +2863,4 @@ vim_SelFile(toplevel, prompt, init_path, show_entry, x, y, fg, bg)
 	}
     }
 }
-
-#endif /* USE_BROWSE */
+#endif /* FEAT_BROWSE */
