@@ -39,6 +39,9 @@
  */
 # define rb_cFalseClass		(*dll_rb_cFalseClass)
 # define rb_cFixnum		(*dll_rb_cFixnum)
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 20
+#  define rb_cFloat		(*dll_rb_cFloat)
+# endif
 # define rb_cNilClass		(*dll_rb_cNilClass)
 # define rb_cSymbol		(*dll_rb_cSymbol)
 # define rb_cTrueClass		(*dll_rb_cTrueClass)
@@ -85,6 +88,20 @@
 # define rb_int2big rb_int2big_stub
 #endif
 
+#if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 19 \
+	&& VIM_SIZEOF_INT < VIM_SIZEOF_LONG
+/* Ruby 1.9 defines a number of static functions which use rb_fix2int and
+ * rb_num2int if VIM_SIZEOF_INT < VIM_SIZEOF_LONG (64bit) */
+# define rb_fix2int rb_fix2int_stub
+# define rb_num2int rb_num2int_stub
+#endif
+
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 21
+/* Ruby 2.1 adds new GC called RGenGC and RARRAY_PTR uses
+ * rb_gc_writebarrier_unprotect_promoted if USE_RGENGC  */
+#  define rb_gc_writebarrier_unprotect_promoted rb_gc_writebarrier_unprotect_promoted_stub
+# endif
+
 #include <ruby.h>
 #ifdef RUBY19_OR_LATER
 # include <ruby/encoding.h>
@@ -102,7 +119,7 @@
 #endif
 
 /*
- * Backward compatiblity for Ruby 1.8 and earlier.
+ * Backward compatibility for Ruby 1.8 and earlier.
  * Ruby 1.9 does not provide STR2CSTR, instead StringValuePtr is provided.
  * Ruby 1.9 does not provide RXXX(s)->len and RXXX(s)->ptr, instead
  * RXXX_LEN(s) and RXXX_PTR(s) are provided.
@@ -133,6 +150,7 @@
 #endif
 
 static int ruby_initialized = 0;
+static void *ruby_stack_start;
 static VALUE objtbl;
 
 static VALUE mVIM;
@@ -146,95 +164,118 @@ static void error_print(int);
 static void ruby_io_init(void);
 static void ruby_vim_init(void);
 
-#if defined(DYNAMIC_RUBY) || defined(PROTO)
-#ifdef PROTO
-# define HINSTANCE int		/* for generating prototypes */
+#if defined(RUBY19_OR_LATER) || defined(RUBY_INIT_STACK)
+# if defined(__ia64) && !defined(ruby_init_stack)
+#  define ruby_init_stack(addr) ruby_init_stack((addr), rb_ia64_bsp())
+# endif
 #endif
 
+#if defined(DYNAMIC_RUBY) || defined(PROTO)
+# ifdef PROTO
+#  define HINSTANCE int		/* for generating prototypes */
+# endif
+
 /*
  * Wrapper defines
  */
-#define rb_assoc_new			dll_rb_assoc_new
-#define rb_cObject			(*dll_rb_cObject)
-#define rb_check_type			dll_rb_check_type
-#define rb_class_path			dll_rb_class_path
-#define rb_data_object_alloc		dll_rb_data_object_alloc
-#define rb_define_class_under		dll_rb_define_class_under
-#define rb_define_const			dll_rb_define_const
-#define rb_define_global_function	dll_rb_define_global_function
-#define rb_define_method		dll_rb_define_method
-#define rb_define_module		dll_rb_define_module
-#define rb_define_module_function	dll_rb_define_module_function
-#define rb_define_singleton_method	dll_rb_define_singleton_method
-#define rb_define_virtual_variable	dll_rb_define_virtual_variable
-#define rb_stdout			(*dll_rb_stdout)
-#define rb_eArgError			(*dll_rb_eArgError)
-#define rb_eIndexError			(*dll_rb_eIndexError)
-#define rb_eRuntimeError		(*dll_rb_eRuntimeError)
-#define rb_eStandardError		(*dll_rb_eStandardError)
-#define rb_eval_string_protect		dll_rb_eval_string_protect
-#define rb_global_variable		dll_rb_global_variable
-#define rb_hash_aset			dll_rb_hash_aset
-#define rb_hash_new			dll_rb_hash_new
-#define rb_inspect			dll_rb_inspect
-#define rb_int2inum			dll_rb_int2inum
-#define rb_lastline_get			dll_rb_lastline_get
-#define rb_lastline_set			dll_rb_lastline_set
-#define rb_load_protect			dll_rb_load_protect
-#ifndef RUBY19_OR_LATER
-#define rb_num2long			dll_rb_num2long
-#endif
-#define rb_num2ulong			dll_rb_num2ulong
-#define rb_obj_alloc			dll_rb_obj_alloc
-#define rb_obj_as_string		dll_rb_obj_as_string
-#define rb_obj_id			dll_rb_obj_id
-#define rb_raise			dll_rb_raise
-#define rb_str_cat			dll_rb_str_cat
-#define rb_str_concat			dll_rb_str_concat
-#define rb_str_new			dll_rb_str_new
-#ifdef rb_str_new2
+# define rb_assoc_new			dll_rb_assoc_new
+# define rb_cObject			(*dll_rb_cObject)
+# define rb_check_type			dll_rb_check_type
+# define rb_class_path			dll_rb_class_path
+# define rb_data_object_alloc		dll_rb_data_object_alloc
+# define rb_define_class_under		dll_rb_define_class_under
+# define rb_define_const			dll_rb_define_const
+# define rb_define_global_function	dll_rb_define_global_function
+# define rb_define_method		dll_rb_define_method
+# define rb_define_module		dll_rb_define_module
+# define rb_define_module_function	dll_rb_define_module_function
+# define rb_define_singleton_method	dll_rb_define_singleton_method
+# define rb_define_virtual_variable	dll_rb_define_virtual_variable
+# define rb_stdout			(*dll_rb_stdout)
+# define rb_eArgError			(*dll_rb_eArgError)
+# define rb_eIndexError			(*dll_rb_eIndexError)
+# define rb_eRuntimeError		(*dll_rb_eRuntimeError)
+# define rb_eStandardError		(*dll_rb_eStandardError)
+# define rb_eval_string_protect		dll_rb_eval_string_protect
+# define rb_global_variable		dll_rb_global_variable
+# define rb_hash_aset			dll_rb_hash_aset
+# define rb_hash_new			dll_rb_hash_new
+# define rb_inspect			dll_rb_inspect
+# define rb_int2inum			dll_rb_int2inum
+# if VIM_SIZEOF_INT < VIM_SIZEOF_LONG /* 64 bits only */
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER <= 18
+#   define rb_fix2int			dll_rb_fix2int
+#   define rb_num2int			dll_rb_num2int
+#  endif
+#  define rb_num2uint			dll_rb_num2uint
+# endif
+# define rb_lastline_get			dll_rb_lastline_get
+# define rb_lastline_set			dll_rb_lastline_set
+# define rb_load_protect			dll_rb_load_protect
+# ifndef RUBY19_OR_LATER
+#  define rb_num2long			dll_rb_num2long
+# endif
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER <= 19
+#  define rb_num2ulong			dll_rb_num2ulong
+# endif
+# define rb_obj_alloc			dll_rb_obj_alloc
+# define rb_obj_as_string		dll_rb_obj_as_string
+# define rb_obj_id			dll_rb_obj_id
+# define rb_raise			dll_rb_raise
+# define rb_str_cat			dll_rb_str_cat
+# define rb_str_concat			dll_rb_str_concat
+# define rb_str_new			dll_rb_str_new
+# ifdef rb_str_new2
 /* Ruby may #define rb_str_new2 to use rb_str_new_cstr. */
-# define need_rb_str_new_cstr 1
+#  define need_rb_str_new_cstr 1
 /* Ruby's headers #define rb_str_new_cstr to make use of GCC's
  * __builtin_constant_p extension. */
-# undef rb_str_new_cstr
-# define rb_str_new_cstr		dll_rb_str_new_cstr
-#else
-# define rb_str_new2			dll_rb_str_new2
-#endif
-#if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
-# define rb_string_value		dll_rb_string_value
-# define rb_string_value_ptr		dll_rb_string_value_ptr
-# define rb_float_new			dll_rb_float_new
-# define rb_ary_new			dll_rb_ary_new
-# define rb_ary_push			dll_rb_ary_push
-#else
-# define rb_str2cstr			dll_rb_str2cstr
-#endif
-#ifdef RUBY19_OR_LATER
-# define rb_errinfo			dll_rb_errinfo
-#else
-# define ruby_errinfo			(*dll_ruby_errinfo)
-#endif
-#define ruby_init			dll_ruby_init
-#define ruby_init_loadpath		dll_ruby_init_loadpath
-#ifdef WIN3264
-# define NtInitialize			dll_NtInitialize
+#  undef rb_str_new_cstr
+#  define rb_str_new_cstr		dll_rb_str_new_cstr
+# else
+#  define rb_str_new2			dll_rb_str_new2
+# endif
 # if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
-#  define rb_w32_snprintf		dll_rb_w32_snprintf
+#  define rb_string_value		dll_rb_string_value
+#  define rb_string_value_ptr		dll_rb_string_value_ptr
+#  define rb_float_new			dll_rb_float_new
+#  define rb_ary_new			dll_rb_ary_new
+#  define rb_ary_push			dll_rb_ary_push
+#  if defined(RUBY19_OR_LATER) || defined(RUBY_INIT_STACK)
+#   ifdef __ia64
+#    define rb_ia64_bsp			dll_rb_ia64_bsp
+#    undef ruby_init_stack
+#    define ruby_init_stack(addr)	dll_ruby_init_stack((addr), rb_ia64_bsp())
+#   else
+#    define ruby_init_stack		dll_ruby_init_stack
+#   endif
+#  endif
+# else
+#  define rb_str2cstr			dll_rb_str2cstr
+# endif
+# ifdef RUBY19_OR_LATER
+#  define rb_errinfo			dll_rb_errinfo
+# else
+#  define ruby_errinfo			(*dll_ruby_errinfo)
+# endif
+# define ruby_init			dll_ruby_init
+# define ruby_init_loadpath		dll_ruby_init_loadpath
+# ifdef WIN3264
+#  define NtInitialize			dll_NtInitialize
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+#   define rb_w32_snprintf		dll_rb_w32_snprintf
+#  endif
 # endif
-#endif
 
-#ifdef RUBY19_OR_LATER
-# define ruby_script			dll_ruby_script
-# define rb_enc_find_index		dll_rb_enc_find_index
-# define rb_enc_find			dll_rb_enc_find
-# define rb_enc_str_new			dll_rb_enc_str_new
-# define rb_sprintf			dll_rb_sprintf
-# define rb_require			dll_rb_require
-# define ruby_init_stack		dll_ruby_init_stack
-# define ruby_process_options		dll_ruby_process_options
-#endif
+# ifdef RUBY19_OR_LATER
+#  define ruby_script			dll_ruby_script
+#  define rb_enc_find_index		dll_rb_enc_find_index
+#  define rb_enc_find			dll_rb_enc_find
+#  define rb_enc_str_new			dll_rb_enc_str_new
+#  define rb_sprintf			dll_rb_sprintf
+#  define rb_require			dll_rb_require
+#  define ruby_process_options		dll_ruby_process_options
+# endif
 
 /*
  * Pointers for dynamic link
@@ -242,6 +283,9 @@ static void ruby_vim_init(void);
 static VALUE (*dll_rb_assoc_new) (VALUE, VALUE);
 VALUE *dll_rb_cFalseClass;
 VALUE *dll_rb_cFixnum;
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 20
+VALUE *dll_rb_cFloat;
+# endif
 VALUE *dll_rb_cNilClass;
 static VALUE *dll_rb_cObject;
 VALUE *dll_rb_cSymbol;
@@ -268,7 +312,11 @@ static VALUE (*dll_rb_hash_aset) (VALUE, VALUE, VALUE);
 static VALUE (*dll_rb_hash_new) (void);
 static VALUE (*dll_rb_inspect) (VALUE);
 static VALUE (*dll_rb_int2inum) (long);
-static VALUE (*dll_rb_int2inum) (long);
+# if VIM_SIZEOF_INT < VIM_SIZEOF_LONG /* 64 bits only */
+static long (*dll_rb_fix2int) (VALUE);
+static long (*dll_rb_num2int) (VALUE);
+static unsigned long (*dll_rb_num2uint) (VALUE);
+# endif
 static VALUE (*dll_rb_lastline_get) (void);
 static void (*dll_rb_lastline_set) (VALUE);
 static void (*dll_rb_load_protect) (VALUE, int, int*);
@@ -278,55 +326,66 @@ static VALUE (*dll_rb_obj_alloc) (VALUE);
 static VALUE (*dll_rb_obj_as_string) (VALUE);
 static VALUE (*dll_rb_obj_id) (VALUE);
 static void (*dll_rb_raise) (VALUE, const char*, ...);
-#if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
 static VALUE (*dll_rb_string_value) (volatile VALUE*);
-#else
+# else
 static char *(*dll_rb_str2cstr) (VALUE,int*);
-#endif
+# endif
 static VALUE (*dll_rb_str_cat) (VALUE, const char*, long);
 static VALUE (*dll_rb_str_concat) (VALUE, VALUE);
 static VALUE (*dll_rb_str_new) (const char*, long);
-#ifdef need_rb_str_new_cstr
+# ifdef need_rb_str_new_cstr
 /* Ruby may #define rb_str_new2 to use rb_str_new_cstr. */
 static VALUE (*dll_rb_str_new_cstr) (const char*);
-#else
+# else
 static VALUE (*dll_rb_str_new2) (const char*);
-#endif
-#ifdef RUBY19_OR_LATER
+# endif
+# ifdef RUBY19_OR_LATER
 static VALUE (*dll_rb_errinfo) (void);
-#else
+# else
 static VALUE *dll_ruby_errinfo;
-#endif
+# endif
 static void (*dll_ruby_init) (void);
 static void (*dll_ruby_init_loadpath) (void);
-#ifdef WIN3264
+# ifdef WIN3264
 static void (*dll_NtInitialize) (int*, char***);
-# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
 static int (*dll_rb_w32_snprintf)(char*, size_t, const char*, ...);
+#  endif
 # endif
-#endif
-#if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
 static char * (*dll_rb_string_value_ptr) (volatile VALUE*);
 static VALUE (*dll_rb_float_new) (double);
 static VALUE (*dll_rb_ary_new) (void);
 static VALUE (*dll_rb_ary_push) (VALUE, VALUE);
-#endif
-#ifdef RUBY19_OR_LATER
+#  if defined(RUBY19_OR_LATER) || defined(RUBY_INIT_STACK)
+#   ifdef __ia64
+static void * (*dll_rb_ia64_bsp) (void);
+static void (*dll_ruby_init_stack)(VALUE*, void*);
+#   else
+static void (*dll_ruby_init_stack)(VALUE*);
+#   endif
+#  endif
+# endif
+# ifdef RUBY19_OR_LATER
 static VALUE (*dll_rb_int2big)(SIGNED_VALUE);
-#endif
+# endif
 
-#ifdef RUBY19_OR_LATER
+# ifdef RUBY19_OR_LATER
 static void (*dll_ruby_script) (const char*);
 static int (*dll_rb_enc_find_index) (const char*);
 static rb_encoding* (*dll_rb_enc_find) (const char*);
 static VALUE (*dll_rb_enc_str_new) (const char*, long, rb_encoding*);
 static VALUE (*dll_rb_sprintf) (const char*, ...);
 static VALUE (*dll_rb_require) (const char*);
-static void (*ruby_init_stack)(VALUE*);
 static void* (*ruby_process_options)(int, char**);
-#endif
+# endif
 
-#ifdef RUBY19_OR_LATER
+# if defined(USE_RGENGC) && USE_RGENGC
+static void (*dll_rb_gc_writebarrier_unprotect_promoted)(VALUE);
+# endif
+
+# if defined(RUBY19_OR_LATER) && !defined(PROTO)
 SIGNED_VALUE rb_num2long_stub(VALUE x)
 {
     return dll_rb_num2long(x);
@@ -335,7 +394,36 @@ VALUE rb_int2big_stub(SIGNED_VALUE x)
 {
     return dll_rb_int2big(x);
 }
-#endif
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 19 \
+	&& VIM_SIZEOF_INT < VIM_SIZEOF_LONG
+long rb_fix2int_stub(VALUE x)
+{
+    return dll_rb_fix2int(x);
+}
+long rb_num2int_stub(VALUE x)
+{
+    return dll_rb_num2int(x);
+}
+#  endif
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 20
+VALUE
+rb_float_new_in_heap(double d)
+{
+    return dll_rb_float_new(d);
+}
+VALUE rb_num2ulong(VALUE x)
+{
+    return (long)RSHIFT((SIGNED_VALUE)(x),1);
+}
+#  endif
+# endif
+
+# if defined(USE_RGENGC) && USE_RGENGC
+void rb_gc_writebarrier_unprotect_promoted_stub(VALUE obj)
+{
+    return dll_rb_gc_writebarrier_unprotect_promoted(obj);
+}
+# endif
 
 static HINSTANCE hinstRuby = NULL; /* Instance of ruby.dll */
 
@@ -351,6 +439,9 @@ static struct
     {"rb_assoc_new", (RUBY_PROC*)&dll_rb_assoc_new},
     {"rb_cFalseClass", (RUBY_PROC*)&dll_rb_cFalseClass},
     {"rb_cFixnum", (RUBY_PROC*)&dll_rb_cFixnum},
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 20
+    {"rb_cFloat", (RUBY_PROC*)&dll_rb_cFloat},
+# endif
     {"rb_cNilClass", (RUBY_PROC*)&dll_rb_cNilClass},
     {"rb_cObject", (RUBY_PROC*)&dll_rb_cObject},
     {"rb_cSymbol", (RUBY_PROC*)&dll_rb_cSymbol},
@@ -377,6 +468,11 @@ static struct
     {"rb_hash_new", (RUBY_PROC*)&dll_rb_hash_new},
     {"rb_inspect", (RUBY_PROC*)&dll_rb_inspect},
     {"rb_int2inum", (RUBY_PROC*)&dll_rb_int2inum},
+# if VIM_SIZEOF_INT < VIM_SIZEOF_LONG /* 64 bits only */
+    {"rb_fix2int", (RUBY_PROC*)&dll_rb_fix2int},
+    {"rb_num2int", (RUBY_PROC*)&dll_rb_num2int},
+    {"rb_num2uint", (RUBY_PROC*)&dll_rb_num2uint},
+# endif
     {"rb_lastline_get", (RUBY_PROC*)&dll_rb_lastline_get},
     {"rb_lastline_set", (RUBY_PROC*)&dll_rb_lastline_set},
     {"rb_load_protect", (RUBY_PROC*)&dll_rb_load_protect},
@@ -386,45 +482,49 @@ static struct
     {"rb_obj_as_string", (RUBY_PROC*)&dll_rb_obj_as_string},
     {"rb_obj_id", (RUBY_PROC*)&dll_rb_obj_id},
     {"rb_raise", (RUBY_PROC*)&dll_rb_raise},
-#if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
     {"rb_string_value", (RUBY_PROC*)&dll_rb_string_value},
-#else
+# else
     {"rb_str2cstr", (RUBY_PROC*)&dll_rb_str2cstr},
-#endif
+# endif
     {"rb_str_cat", (RUBY_PROC*)&dll_rb_str_cat},
     {"rb_str_concat", (RUBY_PROC*)&dll_rb_str_concat},
     {"rb_str_new", (RUBY_PROC*)&dll_rb_str_new},
-#ifdef need_rb_str_new_cstr
+# ifdef need_rb_str_new_cstr
     {"rb_str_new_cstr", (RUBY_PROC*)&dll_rb_str_new_cstr},
-#else
+# else
     {"rb_str_new2", (RUBY_PROC*)&dll_rb_str_new2},
-#endif
-#ifdef RUBY19_OR_LATER
+# endif
+# ifdef RUBY19_OR_LATER
     {"rb_errinfo", (RUBY_PROC*)&dll_rb_errinfo},
-#else
+# else
     {"ruby_errinfo", (RUBY_PROC*)&dll_ruby_errinfo},
-#endif
+# endif
     {"ruby_init", (RUBY_PROC*)&dll_ruby_init},
     {"ruby_init_loadpath", (RUBY_PROC*)&dll_ruby_init_loadpath},
-#ifdef WIN3264
+# ifdef WIN3264
     {
-# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER < 19
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER < 19
     "NtInitialize",
-# else
+#  else
     "ruby_sysinit",
-# endif
+#  endif
 			(RUBY_PROC*)&dll_NtInitialize},
-# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+#  if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
     {"rb_w32_snprintf", (RUBY_PROC*)&dll_rb_w32_snprintf},
+#  endif
 # endif
-#endif
-#if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
+# if defined(DYNAMIC_RUBY_VER) && DYNAMIC_RUBY_VER >= 18
     {"rb_string_value_ptr", (RUBY_PROC*)&dll_rb_string_value_ptr},
+#  if DYNAMIC_RUBY_VER <= 19
     {"rb_float_new", (RUBY_PROC*)&dll_rb_float_new},
+#  else
+    {"rb_float_new_in_heap", (RUBY_PROC*)&dll_rb_float_new},
+#  endif
     {"rb_ary_new", (RUBY_PROC*)&dll_rb_ary_new},
     {"rb_ary_push", (RUBY_PROC*)&dll_rb_ary_push},
-#endif
-#ifdef RUBY19_OR_LATER
+# endif
+# ifdef RUBY19_OR_LATER
     {"rb_int2big", (RUBY_PROC*)&dll_rb_int2big},
     {"ruby_script", (RUBY_PROC*)&dll_ruby_script},
     {"rb_enc_find_index", (RUBY_PROC*)&dll_rb_enc_find_index},
@@ -432,9 +532,17 @@ static struct
     {"rb_enc_str_new", (RUBY_PROC*)&dll_rb_enc_str_new},
     {"rb_sprintf", (RUBY_PROC*)&dll_rb_sprintf},
     {"rb_require", (RUBY_PROC*)&dll_rb_require},
-    {"ruby_init_stack", (RUBY_PROC*)&dll_ruby_init_stack},
     {"ruby_process_options", (RUBY_PROC*)&dll_ruby_process_options},
-#endif
+# endif
+# if defined(RUBY19_OR_LATER) || defined(RUBY_INIT_STACK)
+#  ifdef __ia64
+    {"rb_ia64_bsp", (RUBY_PROC*)&dll_rb_ia64_bsp},
+#  endif
+    {"ruby_init_stack", (RUBY_PROC*)&dll_ruby_init_stack},
+# endif
+# if defined(USE_RGENGC) && USE_RGENGC
+    {"rb_gc_writebarrier_unprotect_promoted", (RUBY_PROC*)&dll_rb_gc_writebarrier_unprotect_promoted},
+# endif
     {"", NULL},
 };
 
@@ -657,8 +765,8 @@ static int ensure_ruby_initialized(void)
 	    NtInitialize(&argc, &argv);
 #endif
 	    {
-#ifdef RUBY19_OR_LATER
-		RUBY_INIT_STACK;
+#if defined(RUBY19_OR_LATER) || defined(RUBY_INIT_STACK)
+		ruby_init_stack(ruby_stack_start);
 #endif
 		ruby_init();
 	    }
@@ -1196,21 +1304,21 @@ static VALUE window_set_height(VALUE self, VALUE height)
     return height;
 }
 
-static VALUE window_width(VALUE self)
+static VALUE window_width(VALUE self UNUSED)
 {
-    win_T *win = get_win(self);
-
-    return INT2NUM(win->w_width);
+    return INT2NUM(W_WIDTH(get_win(self)));
 }
 
-static VALUE window_set_width(VALUE self, VALUE width)
+static VALUE window_set_width(VALUE self UNUSED, VALUE width)
 {
+#ifdef FEAT_VERTSPLIT
     win_T *win = get_win(self);
     win_T *savewin = curwin;
 
     curwin = win;
     win_setwidth(NUM2INT(width));
     curwin = savewin;
+#endif
     return width;
 }
 
@@ -1238,6 +1346,11 @@ static VALUE window_set_cursor(VALUE self, VALUE pos)
     return Qnil;
 }
 
+static VALUE f_nop(VALUE self UNUSED)
+{
+    return Qnil;
+}
+
 static VALUE f_p(int argc, VALUE *argv, VALUE self UNUSED)
 {
     int i;
@@ -1259,6 +1372,7 @@ static void ruby_io_init(void)
 
     rb_stdout = rb_obj_alloc(rb_cObject);
     rb_define_singleton_method(rb_stdout, "write", vim_message, 1);
+    rb_define_singleton_method(rb_stdout, "flush", f_nop, 0);
     rb_define_global_function("p", f_p, -1);
 }
 
@@ -1268,7 +1382,7 @@ static void ruby_vim_init(void)
     rb_global_variable(&objtbl);
 
     /* The Vim module used to be called "VIM", but "Vim" is better.  Make an
-     * alias "VIM" for backwards compatiblity. */
+     * alias "VIM" for backwards compatibility. */
     mVIM = rb_define_module("Vim");
     rb_define_const(rb_cObject, "VIM", mVIM);
     rb_define_const(mVIM, "VERSION_MAJOR", INT2NUM(VIM_VERSION_MAJOR));
@@ -1324,3 +1438,9 @@ static void ruby_vim_init(void)
     rb_define_virtual_variable("$curbuf", buffer_s_current, 0);
     rb_define_virtual_variable("$curwin", window_s_current, 0);
 }
+
+void vim_ruby_init(void *stack_start)
+{
+    /* should get machine stack start address early in main function */
+    ruby_stack_start = stack_start;
+}
