@@ -56,8 +56,11 @@ static char_u *menu_skip_part __ARGS((char_u *p));
 #endif
 #ifdef FEAT_MULTI_LANG
 static char_u *menutrans_lookup __ARGS((char_u *name, int len));
+static void menu_unescape_name  __ARGS((char_u	*p));
 #endif
 
+static char_u *menu_translate_tab_and_shift __ARGS((char_u *arg_start));
+
 /* The character for each menu mode */
 static char_u	menu_mode_chars[] = {'n', 'v', 's', 'o', 'i', 'c', 't'};
 
@@ -106,10 +109,6 @@ ex_menu(eap)
     int		pri_tab[MENUDEPTH + 1];
     int		enable = MAYBE;	    /* TRUE for "menu enable", FALSE for "menu
 				     * disable */
-#ifdef FEAT_MULTI_LANG
-    char_u	*tofree = NULL;
-    char_u	*new_cmd;
-#endif
 #ifdef FEAT_TOOLBAR
     char_u	*icon = NULL;
 #endif
@@ -251,41 +250,6 @@ ex_menu(eap)
     }
 #endif
 
-#ifdef FEAT_MULTI_LANG
-    /*
-     * Translate menu names as specified with ":menutrans" commands.
-     */
-    menu_path = arg;
-    while (*menu_path)
-    {
-	/* find the end of one part and check if it should be translated */
-	p = menu_skip_part(menu_path);
-	map_to = menutrans_lookup(menu_path, (int)(p - menu_path));
-	if (map_to != NULL)
-	{
-	    /* found a match: replace with the translated part */
-	    i = (int)STRLEN(map_to);
-	    new_cmd = alloc((unsigned)STRLEN(arg) + i + 1);
-	    if (new_cmd == NULL)
-		break;
-	    mch_memmove(new_cmd, arg, menu_path - arg);
-	    mch_memmove(new_cmd + (menu_path - arg), map_to, (size_t)i);
-	    STRCPY(new_cmd + (menu_path - arg) + i, p);
-	    p = new_cmd + (menu_path - arg) + i;
-	    vim_free(tofree);
-	    tofree = new_cmd;
-	    arg = new_cmd;
-	}
-	if (*p != '.')
-	    break;
-	menu_path = p + 1;
-    }
-#endif
-
-    /*
-     * Isolate the menu name.
-     * Skip the menu name, and translate <Tab> into a real TAB.
-     */
     menu_path = arg;
     if (*menu_path == '.')
     {
@@ -293,21 +257,7 @@ ex_menu(eap)
 	goto theend;
     }
 
-    while (*arg && !vim_iswhite(*arg))
-    {
-	if ((*arg == '\\' || *arg == Ctrl_V) && arg[1] != NUL)
-	    arg++;
-	else if (STRNICMP(arg, "<TAB>", 5) == 0)
-	{
-	    *arg = TAB;
-	    STRMOVE(arg + 1, arg + 5);
-	}
-	arg++;
-    }
-    if (*arg != NUL)
-	*arg++ = NUL;
-    arg = skipwhite(arg);
-    map_to = arg;
+    map_to = menu_translate_tab_and_shift(arg);
 
     /*
      * If there is only a menu name, display menus with that name.
@@ -453,11 +403,7 @@ ex_menu(eap)
 #endif
 
 theend:
-#ifdef FEAT_MULTI_LANG
-    vim_free(tofree);
-#else
     ;
-#endif
 }
 
 /*
@@ -490,6 +436,7 @@ add_menu_path(menu_path, menuarg, pri_tab, call_data
     char_u	*next_name;
     int		i;
     int		c;
+    int		d;
 #ifdef FEAT_GUI
     int		idx;
     int		new_idx;
@@ -497,6 +444,10 @@ add_menu_path(menu_path, menuarg, pri_tab, call_data
     int		pri_idx = 0;
     int		old_modes = 0;
     int		amenu;
+#ifdef FEAT_MULTI_LANG
+    char_u	*en_name;
+    char_u	*map_to = NULL;
+#endif
 
     /* Make a copy so we can stuff around with it, since it could be const */
     path_name = vim_strsave(menu_path);
@@ -510,6 +461,16 @@ add_menu_path(menu_path, menuarg, pri_tab, call_data
 	/* Get name of this element in the menu hierarchy, and the simplified
 	 * name (without mnemonic and accelerator text). */
 	next_name = menu_name_skip(name);
+#ifdef	FEAT_MULTI_LANG
+	map_to = menutrans_lookup(name, (int)STRLEN(name));
+	if (map_to != NULL)
+	{
+	    en_name = name;
+	    name = map_to;
+	}
+	else
+	    en_name = NULL;
+#endif
 	dname = menu_text(name, NULL, NULL);
 	if (dname == NULL)
 	    goto erret;
@@ -593,6 +554,18 @@ add_menu_path(menu_path, menuarg, pri_tab, call_data
 	    menu->name = vim_strsave(name);
 	    /* separate mnemonic and accelerator text from actual menu name */
 	    menu->dname = menu_text(name, &menu->mnemonic, &menu->actext);
+#ifdef	FEAT_MULTI_LANG
+	    if (en_name != NULL)
+	    {
+		menu->en_name = vim_strsave(en_name);
+		menu->en_dname = menu_text(en_name, NULL, NULL);
+	    }
+	    else
+	    {
+		menu->en_name = NULL;
+		menu->en_dname = NULL;
+	    }
+#endif
 	    menu->priority = pri_tab[pri_idx];
 	    menu->parent = parent;
 #ifdef FEAT_GUI_MOTIF
@@ -746,6 +719,7 @@ add_menu_path(menu_path, menuarg, pri_tab, call_data
 		 * Don't do this if adding a tearbar (addtearoff == FALSE).
 		 * Don't do this for "<Nop>". */
 		c = 0;
+		d = 0;
 		if (amenu && call_data != NULL && *call_data != NUL
 #ifdef FEAT_GUI_W32
 		       && addtearoff
@@ -761,18 +735,25 @@ add_menu_path(menu_path, menuarg, pri_tab, call_data
 			    c = Ctrl_C;
 			    break;
 			case MENU_INSERT_MODE:
-			    c = Ctrl_O;
+			    c = Ctrl_BSL;
+			    d = Ctrl_O;
 			    break;
 		    }
 		}
 
-		if (c)
+		if (c != 0)
 		{
-		    menu->strings[i] = alloc((unsigned)(STRLEN(call_data) + 4));
+		    menu->strings[i] = alloc((unsigned)(STRLEN(call_data) + 5 ));
 		    if (menu->strings[i] != NULL)
 		    {
 			menu->strings[i][0] = c;
-			STRCPY(menu->strings[i] + 1, call_data);
+			if (d == 0)
+			    STRCPY(menu->strings[i] + 1, call_data);
+			else
+			{
+			    menu->strings[i][1] = d;
+			    STRCPY(menu->strings[i] + 2, call_data);
+			}
 			if (c == Ctrl_C)
 			{
 			    int	    len = (int)STRLEN(menu->strings[i]);
@@ -1031,6 +1012,10 @@ free_menu(menup)
     *menup = menu->next;
     vim_free(menu->name);
     vim_free(menu->dname);
+#ifdef FEAT_MULTI_LANG
+    vim_free(menu->en_name);
+    vim_free(menu->en_dname);
+#endif
     vim_free(menu->actext);
 #ifdef FEAT_TOOLBAR
     vim_free(menu->iconfile);
@@ -1348,9 +1333,17 @@ get_menu_name(xp, idx)
 {
     static vimmenu_T	*menu = NULL;
     char_u		*str;
+#ifdef FEAT_MULTI_LANG
+    static  int		should_advance = FALSE;
+#endif
 
     if (idx == 0)	    /* first call: start at first item */
+    {
 	menu = expand_menu;
+#ifdef FEAT_MULTI_LANG
+	should_advance = FALSE;
+#endif
+    }
 
     /* Skip PopUp[nvoci]. */
     while (menu != NULL && (menu_is_hidden(menu->dname)
@@ -1363,12 +1356,30 @@ get_menu_name(xp, idx)
 	return NULL;
 
     if (menu->modes & expand_modes)
-	str = menu->dname;
+#ifdef FEAT_MULTI_LANG
+	if (should_advance)
+	    str = menu->en_dname;
+	else
+	{
+#endif
+	    str = menu->dname;
+#ifdef FEAT_MULTI_LANG
+	    if (menu->en_dname == NULL)
+		should_advance = TRUE;
+	}
+#endif
     else
 	str = (char_u *)"";
 
-    /* Advance to next menu entry. */
-    menu = menu->next;
+#ifdef FEAT_MULTI_LANG
+    if (should_advance)
+#endif
+	/* Advance to next menu entry. */
+	menu = menu->next;
+
+#ifdef FEAT_MULTI_LANG
+    should_advance = !should_advance;
+#endif
 
     return str;
 }
@@ -1383,11 +1394,20 @@ get_menu_names(xp, idx)
     int		idx;
 {
     static vimmenu_T	*menu = NULL;
-    static char_u	tbuffer[256]; /*hack*/
+#define TBUFFER_LEN 256
+    static char_u	tbuffer[TBUFFER_LEN]; /*hack*/
     char_u		*str;
+#ifdef FEAT_MULTI_LANG
+    static  int		should_advance = FALSE;
+#endif
 
     if (idx == 0)	    /* first call: start at first item */
+    {
 	menu = expand_menu;
+#ifdef FEAT_MULTI_LANG
+	should_advance = FALSE;
+#endif
+    }
 
     /* Skip Browse-style entries, popup menus and separators. */
     while (menu != NULL
@@ -1407,20 +1427,51 @@ get_menu_names(xp, idx)
     {
 	if (menu->children != NULL)
 	{
-	    STRCPY(tbuffer, menu->dname);
+#ifdef FEAT_MULTI_LANG
+	    if (should_advance)
+		vim_strncpy(tbuffer, menu->en_dname, TBUFFER_LEN - 2);
+	    else
+	    {
+#endif
+		vim_strncpy(tbuffer, menu->dname,  TBUFFER_LEN - 2);
+#ifdef FEAT_MULTI_LANG
+		if (menu->en_dname == NULL)
+		    should_advance = TRUE;
+	    }
+#endif
 	    /* hack on menu separators:  use a 'magic' char for the separator
 	     * so that '.' in names gets escaped properly */
 	    STRCAT(tbuffer, "\001");
 	    str = tbuffer;
 	}
 	else
-	    str = menu->dname;
+#ifdef FEAT_MULTI_LANG
+	{
+	    if (should_advance)
+		str = menu->en_dname;
+	    else
+	    {
+#endif
+		str = menu->dname;
+#ifdef FEAT_MULTI_LANG
+		if (menu->en_dname == NULL)
+		    should_advance = TRUE;
+	    }
+	}
+#endif
     }
     else
 	str = (char_u *)"";
 
-    /* Advance to next menu entry. */
-    menu = menu->next;
+#ifdef FEAT_MULTI_LANG
+    if (should_advance)
+#endif
+	/* Advance to next menu entry. */
+	menu = menu->next;
+
+#ifdef FEAT_MULTI_LANG
+    should_advance = !should_advance;
+#endif
 
     return str;
 }
@@ -1460,7 +1511,13 @@ menu_name_equal(name, menu)
     char_u	*name;
     vimmenu_T	*menu;
 {
-    return (menu_namecmp(name, menu->name) || menu_namecmp(name, menu->dname));
+#ifdef FEAT_MULTI_LANG
+    if (menu->en_name != NULL
+	    && (menu_namecmp(name, menu->en_name)
+		|| menu_namecmp(name, menu->en_dname)))
+	return TRUE;
+#endif
+    return menu_namecmp(name, menu->name) || menu_namecmp(name, menu->dname);
 }
 
     static int
@@ -1583,7 +1640,6 @@ get_menu_index(menu, state)
 	idx = MENU_INDEX_INSERT;
     else if (state & CMDLINE)
 	idx = MENU_INDEX_CMDLINE;
-#ifdef FEAT_VISUAL
     else if (VIsual_active)
     {
 	if (VIsual_select)
@@ -1591,7 +1647,6 @@ get_menu_index(menu, state)
 	else
 	    idx = MENU_INDEX_VISUAL;
     }
-#endif
     else if (state == HITRETURN || state == ASKMORE)
 	idx = MENU_INDEX_CMDLINE;
     else if (finish_op)
@@ -1754,14 +1809,12 @@ menu_is_tearoff(name)
     static int
 get_menu_mode()
 {
-#ifdef FEAT_VISUAL
     if (VIsual_active)
     {
 	if (VIsual_select)
 	    return MENU_INDEX_SELECT;
 	return MENU_INDEX_VISUAL;
     }
-#endif
     if (State & INSERT)
 	return MENU_INDEX_INSERT;
     if ((State & CMDLINE) || State == ASKMORE || State == HITRETURN)
@@ -2286,7 +2339,7 @@ gui_find_menu(path_name)
 
 	while (menu != NULL)
 	{
-	    if (STRCMP(name, menu->name) == 0 || STRCMP(name, menu->dname) == 0)
+	    if (menu_name_equal(name, menu))
 	    {
 		if (menu->children == NULL)
 		{
@@ -2393,6 +2446,10 @@ ex_menutranslate(eap)
 		    to = vim_strnsave(to, (int)(arg - to));
 		    if (from_noamp != NULL && to != NULL)
 		    {
+			menu_translate_tab_and_shift(from);
+			menu_translate_tab_and_shift(to);
+			menu_unescape_name(from);
+			menu_unescape_name(to);
 			tp[menutrans_ga.ga_len].from = from;
 			tp[menutrans_ga.ga_len].from_noamp = from_noamp;
 			tp[menutrans_ga.ga_len].to = to;
@@ -2465,6 +2522,48 @@ menutrans_lookup(name, len)
 
     return NULL;
 }
+
+/*
+ * Unescape the name in the translate dictionary table.
+ */
+    static void
+menu_unescape_name(name)
+    char_u	*name;
+{
+    char_u  *p;
+
+    for (p = name; *p && *p != '.'; mb_ptr_adv(p))
+	if (*p == '\\')
+	    STRMOVE(p, p + 1);
+}
 #endif /* FEAT_MULTI_LANG */
 
+/*
+ * Isolate the menu name.
+ * Skip the menu name, and translate <Tab> into a real TAB.
+ */
+    static char_u *
+menu_translate_tab_and_shift(arg_start)
+    char_u	*arg_start;
+{
+    char_u	*arg = arg_start;
+
+    while (*arg && !vim_iswhite(*arg))
+    {
+	if ((*arg == '\\' || *arg == Ctrl_V) && arg[1] != NUL)
+	    arg++;
+	else if (STRNICMP(arg, "<TAB>", 5) == 0)
+	{
+	    *arg = TAB;
+	    STRMOVE(arg + 1, arg + 5);
+	}
+	arg++;
+    }
+    if (*arg != NUL)
+	*arg++ = NUL;
+    arg = skipwhite(arg);
+
+    return arg;
+}
+
 #endif /* FEAT_MENU */
