@@ -282,6 +282,10 @@ pum_redraw()
     int		round;
     int		n;
 
+    /* Never display more than we have */
+    if (pum_first > pum_size - pum_height)
+	pum_first = pum_size - pum_height;
+
     if (pum_scrollbar)
     {
 	thumb_heigth = pum_height * pum_height / pum_size;
@@ -563,7 +567,9 @@ pum_set_selected(n, repeat)
 	    g_do_tagpreview = 3;
 	    if (p_pvh > 0 && p_pvh < g_do_tagpreview)
 		g_do_tagpreview = p_pvh;
+	    ++RedrawingDisabled;
 	    resized = prepare_tagpreview(FALSE);
+	    --RedrawingDisabled;
 	    g_do_tagpreview = 0;
 
 	    if (curwin->w_p_pvw)
@@ -637,6 +643,12 @@ pum_set_selected(n, repeat)
 
 		    if (curwin != curwin_save && win_valid(curwin_save))
 		    {
+			/* When the first completion is done and the preview
+			 * window is not resized, skip the preview window's
+			 * status line redrawing. */
+			if (ins_compl_active() && !resized)
+			    curwin->w_redr_status = FALSE;
+
 			/* Return cursor to where we were */
 			validate_cursor();
 			redraw_later(SOME_VALID);
@@ -672,10 +684,6 @@ pum_set_selected(n, repeat)
 #endif
     }
 
-    /* Never display more than we have */
-    if (pum_first > pum_size - pum_height)
-	pum_first = pum_size - pum_height;
-
     if (!resized)
 	pum_redraw();
 
